==> xlr-server-23.3.13-sources.jar: com/xebialabs/xlrelease/scheduler/logs/TaskExecutionLogService.scala <==
package com.xebialabs.xlrelease.scheduler.logs

import akka.actor.ActorRef
import com.xebialabs.xlrelease.domain.events.TaskJobExecutedEvent
import com.xebialabs.xlrelease.domain.status.TaskStatus._
import com.xebialabs.xlrelease.events.{EventListener, Subscribe}
import com.xebialabs.xlrelease.repository.Ids
import com.xebialabs.xlrelease.runner.domain.JobId
import com.xebialabs.xlrelease.scheduler.events.JobFinishedEvent
import com.xebialabs.xlrelease.scheduler.logs.ExecutionLogWatchActor._
import com.xebialabs.xlrelease.scheduler.logs.TaskExecutionRepository.{ByTaskId, FinishExecution, UpdateWithLogEntry}
import com.xebialabs.xlrelease.scheduler.storage.spring.StorageConfiguration.URI_SCHEME_LOCAL_STORAGE
import com.xebialabs.xlrelease.service.{BroadcastService, TaskService}
import com.xebialabs.xlrelease.storage.domain.LogEntry
import com.xebialabs.xlrelease.storage.service.StorageService
import com.xebialabs.xlrelease.support.akka.spring.SpringExtension
import com.xebialabs.xlrelease.user.User
import com.xebialabs.xlrelease.views.TaskExecutionLogView
import grizzled.slf4j.Logging
import org.apache.commons.codec.digest.DigestUtils
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Service

import java.io.OutputStream
import java.time.Instant
import scala.jdk.CollectionConverters._

@Service
@EventListener
class TaskExecutionLogService(springExtension: SpringExtension,
                              broadcastService: BroadcastService,
                              storageService: StorageService,
                              taskService: TaskService,
                              taskExecutionRepository: TaskExecutionRepository)
  extends Logging {

  private lazy val executionLogWatcherActorRef: ActorRef = springExtension.actorOf(classOf[ExecutionLogWatcherSupervisor], s"execution-log-watchers")

  def watch(taskId: String, executionId: String): Unit = {
    executionLogWatcherActorRef ! StartWatch(taskId, executionId, User.AUTHENTICATED_USER.getName)
  }

  def stopWatch(executionId: String): Unit = {
    executionLogWatcherActorRef ! StopWatching(executionId, Some(User.AUTHENTICATED_USER.getName))
  }

  def fetch(taskId: String, executionId: String, outputStream: OutputStream, lastJob: JobId, lastChunk: Long): Unit = {
    TaskExecutionLog(taskId, executionId).fetch(storageService, outputStream, lastJob, lastChunk)
  }

  def pong(executionId: String): Unit = {
    executionLogWatcherActorRef ! Pong(executionId)
  }

  def fetchAllExecutions(taskId: String): java.util.List[TaskExecutionLogView] = {
    taskExecutionRepository.find(ByTaskId(taskId), Pageable.unpaged()).getContent.asScala.zipWithIndex.map { case (row, index) =>
      val view = new TaskExecutionLogView()
      view.setId(row.executionId)
      view.setExecutionNo(index + 1)
      view.setLastJob(row.lastJob)
      view.setLastChunk(row.lastChunk)
      val modifiedDate = Option(row.lastModifiedDate) match {
        case Some(date) => date.toEpochMilli
        case None => Instant.now().toEpochMilli
      }
      view.setModifiedDate(modifiedDate)
      Option(row.endDate).foreach(date => view.setEndDate(date.toEpochMilli))
      view
    }.asJava
  }

  def log(logEntry: LogEntry): Unit = {
    val workerLogEntry = logEntry.copy(uriScheme = URI_SCHEME_LOCAL_STORAGE)
    val storedEntryUri = storageService.store(workerLogEntry)
    val taskStatus = taskService.getStatus(logEntry.taskId)
    logger.debug(s"Adding log entry: $logEntry, with task status: $taskStatus")
    //TODO: A new executionId is generated for abort script and failure handler script.
    // While it is not applicable for container based task, it would be nice to handle that here and avoid setting end-date for
    // ABORT_SCRIPT_IN_PROGRESS or FAILURE_HANDLER_IN_PROGRESS status.
    if (taskStatus.isOneOf(COMPLETED, SKIPPED, ABORTED, FAILED, FAILING, ABORT_SCRIPT_IN_PROGRESS, FAILURE_HANDLER_IN_PROGRESS)) {
      taskExecutionRepository.update(UpdateWithLogEntry(workerLogEntry, Some(Instant.now())))
    } else {
      taskExecutionRepository.update(UpdateWithLogEntry(workerLogEntry, None))
    }
    broadcastService.broadcast(TaskLogCreated(logEntry.taskId, logEntry.executionId, storedEntryUri), true)
  }

  def getTaskExecutionEntry(taskId: String, executionId: String): Option[TaskExecutionEntry] = {
    taskExecutionRepository.read(taskId, executionId)
  }

  @Subscribe
  def onJobFinished(event: JobFinishedEvent): Unit = {
    val jobId = event.jobId
    val executionId = event.executionId
    logger.debug(s"finishing job $jobId")
    executionLogWatcherActorRef ! Check(executionId)
  }

  @Subscribe
  def onTaskLogCreated(taskLogEvent: TaskLogCreated): Unit = {
    logger.debug(s"processing log event $taskLogEvent")
    executionLogWatcherActorRef ! NewEntry(taskLogEvent.executionId, taskLogEvent.uri)
  }

  @Subscribe
  def onTaskExecutionDone(event: TaskJobExecutedEvent): Unit = {
    val finishExecution = FinishExecution(event.taskId, event.executionId, endDate = Instant.now())
    taskExecutionRepository.update(finishExecution)
  }
}

object TaskExecutionLogService {
  // TODO hash comes from TaskPersistence, move it to some more common place (domain, utils...)
  def hash(taskId: String): String = DigestUtils.sha256Hex(Ids.getFolderlessId(taskId))
}

==> xlr-server-24.1.4-sources.jar: com/xebialabs/xlrelease/scheduler/logs/TaskExecutionLogService.scala <==
package com.xebialabs.xlrelease.scheduler.logs

import com.xebialabs.xlrelease.domain.events.TaskJobExecutedEvent
import com.xebialabs.xlrelease.domain.status.TaskStatus._
import com.xebialabs.xlrelease.events.{EventListener, Subscribe}
import com.xebialabs.xlrelease.repository.Ids
import com.xebialabs.xlrelease.runner.domain.JobId
import com.xebialabs.xlrelease.scheduler.events.JobFinishedEvent
import com.xebialabs.xlrelease.scheduler.logs.ExecutionLogWatchActor._
import com.xebialabs.xlrelease.scheduler.logs.TaskExecutionRepository.{ByTaskId, FinishExecution, UpdateWithLogEntry}
import com.xebialabs.xlrelease.scheduler.storage.spring.StorageConfiguration.URI_SCHEME_LOCAL_STORAGE
import com.xebialabs.xlrelease.service.{BroadcastService, TaskService}
import com.xebialabs.xlrelease.storage.domain.LogEntry
import com.xebialabs.xlrelease.storage.service.StorageService
import com.xebialabs.xlrelease.support.pekko.spring.SpringExtension
import com.xebialabs.xlrelease.user.User
import com.xebialabs.xlrelease.views.TaskExecutionLogView
import grizzled.slf4j.Logging
import org.apache.commons.codec.digest.DigestUtils
import org.apache.pekko.actor.ActorRef
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Service

import java.io.OutputStream
import java.time.Instant
import scala.jdk.CollectionConverters._

@Service
@EventListener
class TaskExecutionLogService(springExtension: SpringExtension,
                              broadcastService: BroadcastService,
                              storageService: StorageService,
                              taskService: TaskService,
                              taskExecutionRepository: TaskExecutionRepository)
  extends Logging {

  private lazy val executionLogWatcherActorRef: ActorRef = springExtension.actorOf(classOf[ExecutionLogWatcherSupervisor], s"execution-log-watchers")

  def watch(taskId: String, executionId: String): Unit = {
    executionLogWatcherActorRef ! StartWatch(taskId, executionId, User.AUTHENTICATED_USER.getName)
  }

  def stopWatch(executionId: String): Unit = {
    executionLogWatcherActorRef ! StopWatching(executionId, Some(User.AUTHENTICATED_USER.getName))
  }

  def fetch(taskId: String, executionId: String, outputStream: OutputStream, lastJob: JobId, lastChunk: Long): Unit = {
    TaskExecutionLog(taskId, executionId).fetch(storageService, outputStream, lastJob, lastChunk)
  }

  def pong(executionId: String): Unit = {
    executionLogWatcherActorRef ! Pong(executionId)
  }

  def fetchAllExecutions(taskId: String): java.util.List[TaskExecutionLogView] = {
    taskExecutionRepository.find(ByTaskId(taskId), Pageable.unpaged()).getContent.asScala.zipWithIndex.map { case (row, index) =>
      val view = new TaskExecutionLogView()
      view.setId(row.executionId)
      view.setExecutionNo(index + 1)
      view.setLastJob(row.lastJob)
      view.setLastChunk(row.lastChunk)
      val modifiedDate = Option(row.lastModifiedDate) match {
        case Some(date) => date.toEpochMilli
        case None => Instant.now().toEpochMilli
      }
      view.setModifiedDate(modifiedDate)
      Option(row.endDate).foreach(date => view.setEndDate(date.toEpochMilli))
      view
    }.asJava
  }

  def log(logEntry: LogEntry): Unit = {
    val workerLogEntry = logEntry.copy(uriScheme = URI_SCHEME_LOCAL_STORAGE)
    val storedEntryUri = storageService.store(workerLogEntry)
    val taskStatus = taskService.getStatus(logEntry.taskId)
    logger.debug(s"Adding log entry: $logEntry, with task status: $taskStatus")
    //TODO: A new executionId is generated for abort script and failure handler script.
    // While it is not applicable for container based task, it would be nice to handle that here and avoid setting end-date for
    // ABORT_SCRIPT_IN_PROGRESS or FAILURE_HANDLER_IN_PROGRESS status.
    if (taskStatus.isOneOf(COMPLETED, SKIPPED, ABORTED, FAILED, FAILING, ABORT_SCRIPT_IN_PROGRESS, FAILURE_HANDLER_IN_PROGRESS)) {
      taskExecutionRepository.update(UpdateWithLogEntry(workerLogEntry, Some(Instant.now())))
    } else {
      taskExecutionRepository.update(UpdateWithLogEntry(workerLogEntry, None))
    }
    broadcastService.broadcast(TaskLogCreated(logEntry.taskId, logEntry.executionId, storedEntryUri), true)
  }

  def getTaskExecutionEntry(taskId: String, executionId: String): Option[TaskExecutionEntry] = {
    taskExecutionRepository.read(taskId, executionId)
  }

  @Subscribe
  def onJobFinished(event: JobFinishedEvent): Unit = {
    val jobId = event.jobId
    val executionId = event.executionId
    logger.debug(s"finishing job $jobId")
    executionLogWatcherActorRef ! Check(executionId)
  }

  @Subscribe
  def onTaskLogCreated(taskLogEvent: TaskLogCreated): Unit = {
    logger.debug(s"processing log event $taskLogEvent")
    executionLogWatcherActorRef ! NewEntry(taskLogEvent.executionId, taskLogEvent.uri)
  }

  @Subscribe
  def onTaskExecutionDone(event: TaskJobExecutedEvent): Unit = {
    val finishExecution = FinishExecution(event.taskId, event.executionId, endDate = Instant.now())
    taskExecutionRepository.update(finishExecution)
  }
}

object TaskExecutionLogService {
  // TODO hash comes from TaskPersistence, move it to some more common place (domain, utils...)
  def hash(taskId: String): String = DigestUtils.sha256Hex(Ids.getFolderlessId(taskId))
}
